@@ -1,6 +1,6 @@
 // maingui.h
 
-// Copyright (C) 2012-2013 by Werner Lemberg.
+// Copyright (C) 2012-2014 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -44,17 +44,16 @@ class Main_GUI
   Q_OBJECT
 
 public:
-  Main_GUI(int, int, int,
+  Main_GUI(bool,
+           int, int, int,
            bool, bool,
            bool, int,
            const char*,
            bool, bool, bool,
-           bool, bool, const char*, bool,
-           bool);
+           bool, bool, const char*, const char*,
+           bool, bool);
   ~Main_GUI();
 
-  void create_alternative_layout();
-
 protected:
   void closeEvent(QCloseEvent*);
 
@@ -90,11 +89,15 @@ private:
   int pre_hinting;
   int hint_composites;
   int no_info;
+  int default_script_idx;
   int fallback_script_idx;
   int symbol;
   int dehint;
 
-  void create_layout();
+  void create_layout(bool);
+  void create_horizontal_layout();
+  void create_vertical_layout();
+
 
   void create_connections();
   void create_actions();
@@ -129,6 +132,8 @@ private:
   QCheckBox* gdi_box;
   QCheckBox* dw_box;
 
+  QLabel* default_label;
+  QComboBox* default_box;
   QLabel* fallback_label;
   QComboBox* fallback_box;
 
