@@ -1,7 +1,7 @@
 /* tagloadr.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -24,6 +24,10 @@
 #include FT_FREETYPE_H
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct TA_SubGlyphRec_
 {
   FT_Int index;
@@ -124,6 +128,10 @@ FT_Error
 TA_GlyphLoader_CopyPoints(TA_GlyphLoader target,
                           TA_GlyphLoader source);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* __TAGLOADR_H__ */
 
 /* end of tagloadr.h */
