@@ -1,7 +1,7 @@
 /* taglyf.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -1207,13 +1207,16 @@ TA_sfnt_create_glyf_data(SFNT* sfnt,
 }
 
 
-/* While the auto-hinter is glyph oriented (this is, using `glyf' data), */
-/* it relies on the `cmap' table to get script coverage data. */
-/* In TTCs, subfonts normally share the same `glyf' table */
-/* but use different `cmap's.  Covering the most common situation, */
-/* namely a single `glyf' table and multiple `cmap's, */
-/* ttfautohint merges coverage data for the first subfont's `glyf' table */
-/* with all other subfonts that also use this very `glyf' table. */
+/*
+ * While the auto-hinter is glyph oriented (this is, using `glyf' data), it
+ * relies on the `cmap' table and OpenType features to get style coverage
+ * data.  In TTCs, subfonts normally share the same `glyf' table but use
+ * different `cmap's and OpenType features (in `GSUB' and `GPOS' tables).
+ * Covering the most common situation, namely a single `glyf' table and
+ * multiple `cmap's and OpenType features, ttfautohint merges coverage data
+ * for the first subfont's `glyf' table with all other subfonts that also
+ * use this very `glyf' table.
+ */
 
 FT_Error
 TA_sfnt_handle_coverage(SFNT* sfnt,
@@ -1227,13 +1230,13 @@ TA_sfnt_handle_coverage(SFNT* sfnt,
   FT_Face face = sfnt->face;
   TA_FaceGlobals curr_globals;
 
-  TA_Script saved_fallback_script = font->fallback_script;
+  TA_Style saved_fallback_style = font->fallback_style;
 
 
-  /* using TA_SCRIPT_NONE as the fallback script ensures */
+  /* using TA_STYLE_UNASSIGNED as the fallback style ensures */
   /* that uncovered glyphs stay as-is */
-  /* (we handle the fallback script later on) */
-  font->fallback_script = TA_SCRIPT_NONE;
+  /* (we handle the fallback style later on) */
+  font->fallback_style = (TA_Style)TA_STYLE_UNASSIGNED;
 
   /* trigger computation of coverage */
   error = ta_loader_init(font);
@@ -1244,7 +1247,7 @@ TA_sfnt_handle_coverage(SFNT* sfnt,
     goto Exit;
   ta_loader_done(font);
 
-  font->fallback_script = saved_fallback_script;
+  font->fallback_style = saved_fallback_style;
   curr_globals = (TA_FaceGlobals)face->autohint.data;
 
   if (!data->master_globals)
@@ -1260,20 +1263,20 @@ TA_sfnt_handle_coverage(SFNT* sfnt,
     TA_FaceGlobals master_globals = data->master_globals;
     FT_Long count = master_globals->glyph_count;
 
-    FT_Byte* master = master_globals->glyph_scripts;
-    FT_Byte* curr = curr_globals->glyph_scripts;
+    FT_Byte* master = master_globals->glyph_styles;
+    FT_Byte* curr = curr_globals->glyph_styles;
 
     FT_Byte* limit = master + count;
 
 
     /* we simply copy the data, */
     /* assuming that a given glyph always has the same properties -- */
-    /* as soon as we make the script selection more fine-grained, */
+    /* as soon as we make the style selection more fine-grained, */
     /* it is possible that this assumption doesn't hold: */
     /* for example, glyph `A' can be used for both Cyrillic and Latin */
     while (master < limit)
     {
-      if ((*curr & ~TA_DIGIT) != TA_SCRIPT_NONE)
+      if ((*curr & ~TA_DIGIT) != TA_STYLE_UNASSIGNED)
         *master = *curr;
 
       master++;
@@ -1299,21 +1302,55 @@ TA_sfnt_adjust_master_coverage(SFNT* sfnt,
   TA_FaceGlobals curr_globals = (TA_FaceGlobals)face->autohint.data;
 
 
-  /* use fallback script for uncovered glyphs */
+  /* use fallback style for uncovered glyphs */
   if (master_globals == curr_globals)
   {
     FT_Long nn;
-    FT_Byte* gscripts = master_globals->glyph_scripts;
+    FT_Byte* gstyles = master_globals->glyph_styles;
+#ifdef TA_DEBUG
+    FT_UInt count;
+#endif
 
 
     for (nn = 0; nn < master_globals->glyph_count; nn++)
     {
-      if ((gscripts[nn] & ~TA_DIGIT) == TA_SCRIPT_NONE)
+      if ((gstyles[nn] & ~TA_DIGIT) == TA_STYLE_UNASSIGNED)
       {
-        gscripts[nn] &= ~TA_SCRIPT_NONE;
-        gscripts[nn] |= master_globals->font->fallback_script;
+        gstyles[nn] &= ~TA_STYLE_UNASSIGNED;
+        gstyles[nn] |= master_globals->font->fallback_style;
       }
     }
+
+#ifdef TA_DEBUG
+
+    TA_LOG_GLOBAL(("\n"
+                   "using fallback style `%s' for unassigned glyphs:\n",
+                   ta_style_names[master_globals->font->fallback_style]));
+
+    count = 0;
+
+    for (nn = 0; nn < master_globals->glyph_count; nn++)
+    {
+      if ((gstyles[nn] & ~TA_DIGIT) == master_globals->font->fallback_style)
+      {
+        if (!(count % 10))
+          TA_LOG_GLOBAL((" "));
+
+        TA_LOG_GLOBAL((" %d", nn));
+        count++;
+
+        if (!(count % 10))
+          TA_LOG_GLOBAL(("\n"));
+      }
+    }
+
+    if (!count)
+      TA_LOG_GLOBAL(("  (none)\n"));
+    if (count % 10)
+      TA_LOG_GLOBAL(("\n"));
+
+#endif /* TA_DEBUG */
+
     return 1; /* master coverage adjusted */
   }
   else
@@ -1339,8 +1376,8 @@ TA_sfnt_copy_master_coverage(SFNT* sfnt,
   if (master_globals != curr_globals)
   {
     FT_Long count = master_globals->glyph_count;
-    FT_Byte* master = master_globals->glyph_scripts;
-    FT_Byte* curr = curr_globals->glyph_scripts;
+    FT_Byte* master = master_globals->glyph_styles;
+    FT_Byte* curr = curr_globals->glyph_styles;
 
 
     memcpy(curr, master, count);
