@@ -1,7 +1,7 @@
 /* tahints.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -377,7 +377,7 @@ ta_direction_compute(FT_Pos dx,
       ll = -dx;
       ss = dy;
     }
-    }
+  }
   else /* dy < dx */
   {
     if (dy >= -dx)
@@ -454,7 +454,7 @@ ta_glyph_hints_done(TA_GlyphHints hints)
 
 void
 ta_glyph_hints_rescale(TA_GlyphHints hints,
-                       TA_ScriptMetrics metrics)
+                       TA_StyleMetrics metrics)
 {
   hints->metrics = metrics;
   hints->scaler_flags = metrics->scaler.flags;
@@ -785,7 +785,6 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
         in_x = out_x;
         in_y = out_y;
-        prev = point;
       }
     }
   }
@@ -1227,8 +1226,6 @@ ta_glyph_hints_align_weak_points(TA_GlyphHints hints,
     }
   }
 
-  point = points;
-
   for (; contour < contour_limit; contour++)
   {
     TA_Point first_touched, last_touched;
@@ -1251,7 +1248,6 @@ ta_glyph_hints_align_weak_points(TA_GlyphHints hints,
     }
 
     first_touched = point;
-    last_touched = point;
 
     for (;;)
     {
