@@ -1,7 +1,7 @@
 /* talatin.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -28,16 +28,6 @@
 
 extern const TA_WritingSystemClassRec ta_latin_writing_system_class;
 
-/* the latin-specific script classes */
-
-extern const TA_ScriptClassRec ta_cyrl_script_class;
-extern const TA_ScriptClassRec ta_grek_script_class;
-extern const TA_ScriptClassRec ta_hebr_script_class;
-extern const TA_ScriptClassRec ta_latn_script_class;
-#if 0
-extern const TA_ScriptClassRec ta_armn_script_class;
-#endif
-
 
 /* constants are given with units_per_em == 2048 in mind */
 #define TA_LATIN_CONSTANT(metrics, c) \
@@ -93,7 +83,7 @@ typedef struct TA_LatinAxisRec_
 
 typedef struct TA_LatinMetricsRec_
 {
-  TA_ScriptMetricsRec root;
+  TA_StyleMetricsRec root;
   FT_UInt units_per_em;
   TA_LatinAxisRec axis[TA_DIMENSION_MAX];
 } TA_LatinMetricsRec, *TA_LatinMetrics;
@@ -134,7 +124,7 @@ ta_latin_metrics_check_digits(TA_LatinMetrics metrics,
 
 
 /* the next functions shouldn't normally be exported; */
-/* however, other scripts might like to use these functions as-is */
+/* however, other writing systems might like to use these functions as-is */
 
 FT_Error
 ta_latin_hints_compute_segments(TA_GlyphHints hints,
