@@ -1,7 +1,7 @@
 /* ttfautohint.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -91,7 +91,9 @@ TTF_autohint(const char* options,
   FT_Bool symbol = 0;
 
   const char* fallback_script_string = NULL;
-  TA_Script fallback_script = TA_SCRIPT_DFLT;
+  const char* default_script_string = NULL;
+  TA_Style fallback_style = TA_STYLE_NONE_DFLT;
+  TA_Script default_script = TA_SCRIPT_LATN;
 
   FT_Bool dehint = 0;
 
@@ -100,7 +102,7 @@ TTF_autohint(const char* options,
   const char* op;
 
 #undef SCRIPT
-#define SCRIPT(s, S, d) #s,
+#define SCRIPT(s, S, d, h, sc1, sc2, sc3) #s,
 
   const char* script_names[] =
   {
@@ -148,6 +150,8 @@ TTF_autohint(const char* options,
     /* handle options -- don't forget to update parameter dump below! */
     if (COMPARE("debug"))
       debug = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("default-script"))
+      default_script_string = va_arg(ap, const char*);
     else if (COMPARE("dehint"))
       dehint = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("dw-cleartype-strong-stem-width"))
@@ -299,16 +303,42 @@ TTF_autohint(const char* options,
     int i;
 
 
-    for (i = 0; i < TA_SCRIPT_MAX; i++)
-      if (!strcmp(script_names[i], fallback_script_string))
+    for (i = 0; i < TA_STYLE_MAX; i++)
+    {
+      TA_StyleClass style_class = ta_style_classes[i];
+
+
+      if (style_class->coverage == TA_COVERAGE_DEFAULT
+          && !strcmp(script_names[style_class->script],
+                     fallback_script_string))
         break;
+    }
+    if (i == TA_STYLE_MAX)
+    {
+      error = FT_Err_Invalid_Argument;
+      goto Err1;
+    }
+
+    fallback_style = (TA_Style)i;
+  }
+
+  if (default_script_string)
+  {
+    int i;
+
+
+    for (i = 0; i < TA_SCRIPT_MAX; i++)
+    {
+      if (!strcmp(script_names[i], default_script_string))
+        break;
+    }
     if (i == TA_SCRIPT_MAX)
     {
       error = FT_Err_Invalid_Argument;
       goto Err1;
     }
 
-    fallback_script = i;
+    default_script = (TA_Script)i;
   }
 
   if (x_height_snapping_exceptions_string)
@@ -338,7 +368,8 @@ TTF_autohint(const char* options,
   font->ignore_restrictions = ignore_restrictions;
   font->pre_hinting = pre_hinting;
   font->hint_composites = hint_composites;
-  font->fallback_script = fallback_script;
+  font->fallback_style = fallback_style;
+  font->default_script = default_script;
   font->symbol = symbol;
 
 No_check:
@@ -367,10 +398,12 @@ No_check:
       char *s;
 
 
+      DUMPSTR("default-script",
+              script_names[font->default_script]);
       DUMPVAL("dw-cleartype-strong-stem-width",
               font->dw_cleartype_strong_stem_width);
       DUMPSTR("fallback-script",
-              script_names[font->fallback_script]);
+              script_names[ta_style_classes[font->fallback_style]->script]);
       DUMPVAL("gdi-cleartype-strong-stem-width",
               font->gdi_cleartype_strong_stem_width);
       DUMPVAL("gray-strong-stem-width",
@@ -445,7 +478,7 @@ No_check:
 
     /* assure that the font hasn't been already processed by ttfautohint; */
     /* another, more thorough check is done in TA_glyph_parse_simple */
-    idx = FT_Get_Name_Index(sfnt->face, TTFAUTOHINT_GLYPH);
+    idx = FT_Get_Name_Index(sfnt->face, (FT_String*)TTFAUTOHINT_GLYPH);
     if (idx)
     {
       error = TA_Err_Already_Processed;
